************************************************************************
* File (main_cc_trees_vegas.F).
************************************************************************
* This is the main FORTRAN module to calculate Charged Current
* gluon-induced processes at the tree level.
*
* It is produced by s2n package of SANC (v1.10) Project.
* Copyright (C), SANC Project Team, 2002-2008.
*
* http://sanc.jinr.ru/
* http://pcphsanc.cern.ch/
* E-mail: <sanc@jinr.ru>

************************************************************************
* Table of the processes
* [pid - Process IDentification number]:
* -----------------------------------------------------------
* | pid | SANC       | Process                              |
* |     | notation   |                                      |
* -----------------------------------------------------------
* |  7  | 2313141112 | g + up      -> dn      + en   + el^+ |
* |  8  | 1423131211 | g + anti-dn -> anti-up + en   + el^+ |
* |  9  | 2314131211 | g + dn      -> up      + el^- + en   |
* | 10  | 1323141112 | g + anti-up -> anti-dn + el^- + en   |
* | 11  | 2313141516 | g + up      -> dn      + mn   + mo^+ |
* | 12  | 1423131615 | g + anti-dn -> anti-up + mn   + mn^+ |
* | 13  | 2314131615 | g + dn      -> up      + mo^- + mn   |
* | 14  | 1323141516 | g + anti-up -> anti-dn + mo^- + mn   |
* | 15  | 2313141920 | g + up      -> dn      + tn   + ta^+ |
* | 16  | 1423131920 | g + anti-dn -> anti-up + tn   + ta^+ |
* | 17  | 2314132019 | g + dn      -> up      + ta^- + tn   |
* | 18  | 1323142019 | g + anti-up -> anti-dn + ta^- + tn   |
* -----------------------------------------------------------
*
************************************************************************
      program main_cc_trees_vegas
      implicit none!
#include "s2n_declare.h"

      integer*4 ome,lmu,imu,isetup,is,ic,ihard,pid
      integer*4 ncall1,itmx1,itmx2,nrdm
      real*8 sq,sq0,hard_sd
      real*8 costa(5),sa(8),sres(4,10,8)
      common /s/ s
      common /pid/ pid

      data costa/-0.9d0, -0.5d0, 0d0, +0.5d0, +0.9d0/
      data sa/40d0,80d0,120d0,200d0,500d0,1000d0,2000d0,14000d0/
*
*   gfscheme = 0   alpha(0) calculation scheme
*              1   GF
*              2   GF prime
*                
*   ihard    = 1   1spr integration for hard
*              4   4d integration
*
*   isetup   = 0   SANC input [PDG 2006]            CPC 177 (2007) 623
*              1   Les Houches Workshop (2005)      [hep-ph/0604120]
*              2   Tevatron-for-LHC Workshop (2006) [arXiv:0705.3251]
*              3   Les Houches Workshop (2007)      [arXiv:0803.0678]
*

      gfscheme = 0
      ihard = 1
      isetup = 1
      pid = 7

      if (isetup.eq.0) then
#include "sanc_input.h"
         print *, "SANC setup"
      elseif (isetup.eq.1) then
#include "leshw_input.h"
         print *, "Les Houches (2005) setup"
      elseif (isetup.eq.2) then
#include "tev4lhcw_input.h"
         print *, "TeV4LHC (2006) setup"
      elseif (isetup.eq.3) then
#include "leshw2007_input.h"
         print *, "Les Houches (2007) setup"
      endif

      call s2n_init()
      call ffini()

      call ProcessInit (pid)
      call PrintConsts (0)

      nrdm = 12345

      do is=1,1
         sq = sa(is)
         s  = sq**2
         sq0 = mf+mf1
         if (sq.le.sq0) call PrintMessage (1,pid)

         if (ihard.eq.1) then
*     sigma, 1spr
            ncall1 = 50 000
            itmx1 = 10
            itmx2 = 10
            call ProcessHard_1spr (nrdm,ncall1,itmx1,itmx2,hard,hard_sd)
         elseif (ihard.eq.4) then
*     sigma, 4d
            ncall1 = 1 000 000
            itmx1 = 10
            itmx2 = 10
            call ProcessHard (nrdm,ncall1,itmx1,itmx2,hard,hard_sd)
         endif

         print *
         call PrintProcess (pid)
*         call PrintConsts (0)
         print *
         print *, "sq(GeV)        = ", sq
         print *, "[ha](pb)       = ", hard, hard_sd
      enddo

      stop
      end
************************************************************************
      subroutine ProcessHard_1spr (nrdm,ncall1,itmx1,itmx2,hard,hard_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,pid,nrdm
      real*8 sq,hard_sd
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      common /s/ s
      common /pid/ pid
      external function_1spr

      sq = sqrt(s)

      dummy = ranini(nrdm)
      xl(1) = mf**2
      xu(1) = (sq-mf1)**2
      
      ncall = ncall1
      itmx = itmx1
      call vegas (1, function_1spr, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (1, function_1spr, avgi, sd, chi2a)
      hard = avgi
      hard_sd = sd

      return
      end
************************************************************************
      subroutine ProcessHard (nrdm,ncall1,itmx1,itmx2,hard,hard_sd)
      implicit none!
#include "s2n_declare.h"
      integer*4 itmx1,itmx2,ncall1,ncall,itmx,nprn,ndev,pid,nrdm
      real*8 sq,hard_sd
      real*8 xl(10),xu(10),acc,avgi,sd,chi2a,dummy,ranini
      common /bveg1/ ncall, itmx, nprn, ndev, xl, xu, acc
      common /s/ s
      common /pid/ pid
      external function_4d

      sq = sqrt(s)

      xl(1) = mf**2
      xu(1) = (sq-mf1)**2
      xl(2) = 0d0
      xu(2) = 2d0*pi
      xl(3) = -1d0
      xu(3) = +1d0
      xl(4) = -1d0
      xu(4) = +1d0
      
      dummy = ranini(nrdm)
      ncall = ncall1
      itmx = itmx1
      call vegas (4, function_4d, avgi, sd, chi2a)
      itmx = itmx2
      call vegas1 (4, function_4d, avgi, sd, chi2a)
      hard = avgi
      hard_sd = sd

      return
      end
************************************************************************
      real*8 function function_1spr (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 x(10),wgt
      common /s/ s
      common /pid/ pid

      spr = x(1)

      function_1spr = 0d0
      if     (pid.eq.7) then
         call cc_qcd_ha_23_13__14_11_12_1spr (s,spr,hard)
      elseif (pid.eq.8) then
         call cc_qcd_ha_23_14__13_12_11_1spr (s,spr,hard)
      elseif (pid.eq.9) then
         call cc_qcd_ha_13_23__14_11_12_1spr (s,spr,hard)
      elseif (pid.eq.10) then
         call cc_qcd_ha_14_23__13_12_11_1spr (s,spr,hard)
      elseif (pid.eq.11) then
         call cc_qcd_ha_23_13__14_15_16_1spr (s,spr,hard)
      elseif (pid.eq.12) then
         call cc_qcd_ha_23_14__13_16_15_1spr (s,spr,hard)
      elseif (pid.eq.13) then
         call cc_qcd_ha_13_23__14_15_16_1spr (s,spr,hard)
      elseif (pid.eq.14) then
         call cc_qcd_ha_14_23__13_16_15_1spr (s,spr,hard)
      elseif (pid.eq.15) then
         call cc_qcd_ha_23_13__14_19_20_1spr (s,spr,hard)
      elseif (pid.eq.16) then
         call cc_qcd_ha_23_14__13_20_19_1spr (s,spr,hard)
      elseif (pid.eq.17) then
         call cc_qcd_ha_13_23__14_19_20_1spr (s,spr,hard)
      elseif (pid.eq.18) then
         call cc_qcd_ha_14_23__13_20_19_1spr (s,spr,hard)
      endif
      function_1spr = hard
      
      return
      end
************************************************************************
      real*8 function function_4d (x,wgt)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid
      real*8 x(10),wgt,ph3,cth3,cth5,born_ud,born_du
      common /pid/ pid
      common /s/ s

      spr = x(1)
      ph3 = x(2)
      cth3 = x(3)
      cth5 = x(4)

      function_4d = 0d0
      if     (pid.eq.7) then
         call cc_qcd_ha_23_13__14_11_12 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.8) then
         call cc_qcd_ha_23_14__13_12_11 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.9) then
         call cc_qcd_ha_13_23__14_11_12 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.10) then
         call cc_qcd_ha_14_23__13_12_11 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.11) then
         call cc_qcd_ha_23_13__14_15_16 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.12) then
         call cc_qcd_ha_23_14__13_16_15 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.13) then
         call cc_qcd_ha_13_23__14_15_16 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.14) then
         call cc_qcd_ha_14_23__13_16_15 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.15) then
         call cc_qcd_ha_23_13__14_19_20 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.16) then
         call cc_qcd_ha_23_14__13_20_19 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.17) then
         call cc_qcd_ha_13_23__14_19_20 (s,spr,ph3,cth3,cth5,hard)
      elseif (pid.eq.18) then
         call cc_qcd_ha_14_23__13_20_19 (s,spr,ph3,cth3,cth5,hard)
      endif
      function_4d = hard

      return
      end
************************************************************************
      subroutine ProcessInit (pid)
      implicit none!
#include "s2n_declare.h"
      integer*4 pid

      if  ((pid.ne. 7).and.(pid.ne. 8).and.(pid.ne. 9).and.
     &     (pid.ne.10).and.(pid.ne.11).and.(pid.ne.12).and.
     &     (pid.ne.13).and.(pid.ne.14).and.(pid.ne.15).and.
     &     (pid.ne.16).and.(pid.ne.17).and.(pid.ne.18))
     &     call PrintMessage (1,pid)

      print *
      call PrintProcess (pid)
      print *

      if     (pid.eq.7) then
         print *, "mup(GeV) =", mup
         print *, "mdn(GeV) =", mdn
         print *, "mel(GeV) =", mel
         nc = 1d0
         fc = 3d0
         mf  = mel
         mf1 = mdn
      elseif (pid.eq.8) then
         print *, "mup(GeV) =", mup
         print *, "mdn(GeV) =", mdn
         print *, "mel(GeV) =", mel
         nc = 1d0
         fc = 3d0
         mf  = mel
         mf1 = mup
      elseif (pid.eq.9) then
         print *, "mup(GeV) =", mup
         print *, "mdn(GeV) =", mdn
         print *, "mel(GeV) =", mel
         nc = 1d0
         fc = 3d0
         mf  = mel
         mf1 = mdn
      elseif (pid.eq.10) then
         print *, "mup(GeV) =", mup
         print *, "mdn(GeV) =", mdn
         print *, "mel(GeV) =", mel
         nc = 1d0
         fc = 3d0
         mf  = mel
         mf1 = mup
      elseif (pid.eq.11) then
         print *, "mup(GeV) =", mup
         print *, "mdn(GeV) =", mdn
         print *, "mmo(GeV) =", mmo
         nc = 1d0
         fc = 3d0
         mf  = mmo
         mf1 = mdn
      elseif (pid.eq.12) then
         print *, "mup(GeV) =", mup
         print *, "mdn(GeV) =", mdn
         print *, "mmo(GeV) =", mmo
         nc = 1d0
         fc = 3d0
         mf  = mmo
         mf1 = mup
      elseif (pid.eq.13) then
         print *, "mup(GeV) =", mup
         print *, "mdn(GeV) =", mdn
         print *, "mmo(GeV) =", mmo
         nc = 1d0
         fc = 3d0
         mf  = mmo
         mf1 = mdn
      elseif (pid.eq.14) then
         print *, "mup(GeV) =", mup
         print *, "mdn(GeV) =", mdn
         print *, "mmo(GeV) =", mmo
         nc = 1d0
         fc = 3d0
         mf  = mmo
         mf1 = mup
      elseif (pid.eq.15) then
         print *, "mup(GeV) =", mup
         print *, "mdn(GeV) =", mdn
         print *, "mta(GeV) =", mta
         nc = 1d0
         fc = 3d0
         mf  = mta
         mf1 = mdn
      elseif (pid.eq.16) then
         print *, "mup(GeV) =", mup
         print *, "mdn(GeV) =", mdn
         print *, "mta(GeV) =", mta
         nc = 1d0
         fc = 3d0
         mf  = mta
         mf1 = mup
      elseif (pid.eq.17) then
         print *, "mup(GeV) =", mup
         print *, "mdn(GeV) =", mdn
         print *, "mta(GeV) =", mta
         nc = 1d0
         fc = 3d0
         mf  = mta
         mf1 = mdn
      elseif (pid.eq.18) then
         print *, "mup(GeV) =", mup
         print *, "mdn(GeV) =", mdn
         print *, "mta(GeV) =", mta
         nc = 1d0
         fc = 3d0
         mf  = mta
         mf1 = mup
      endif

      return
      end
************************************************************************
      subroutine PrintMessage (pmg,pid)
      implicit none!
#include "s2n_declare.h"
      integer*4 pmg,pid
      real*8 sq0
      common /s/ s

      sq0 = mf+mf1

      if (pmg.eq.1)
     &     print *, "Fatal error, invalid process number: pid = ", pid
      if (pmg.eq.2)
     &     print *, "Fatal error, process under threshold: sq = ",
     &     sqrt(s),"  <  (Mf1+Mf2) = ", sq0 

      stop
      end
************************************************************************
      subroutine PrintConsts (pmg)
      implicit none!
      integer pmg
#include "s2n_declare.h"

      if ((pmg.eq.0).or.(pmg.eq.1)) then
         print *
*         print *, "flag iqed     = ",iqed
*         print *, "flag iew      = ",iew
*         print *, "flag qcd      = ",iqcd
         print *, "flag gfscheme = ",gfscheme
*         print *, "flag ilin     = ",ilin
      endif
      if ((pmg.eq.0).or.(pmg.eq.2)) then
         print *
         print *, "alphai    = ",alphai
         print *, "alpha     = ",alpha
         print *, "GF(GeV-^2)= ",gf
         print *, "g         = ",g
         print *, "e         = ",e
         print *, "alphas    = ",alpha
         print *, "gs        = ",gs
         print *, "cwm       = ",ctw
         print *, "swm2      = ",stw2
         print *, "mh(GeV)   = ",mh
         print *, "mw(GeV)   = ",mw
         print *, "mz(GeV)   = ",mz
         print *, "wz(GeV)   = ",wz
         print *, "ww(GeV)   = ",ww
         print *, "wh(GeV)   = ",wh
         print *, "wtp(GeV)  = ",wtp
      endif

      return
      end
************************************************************************
      subroutine PrintProcess (pid)
      implicit none!
      integer*4 pid

      if (pid.eq. 7) print *,
     &  "pid = ", pid, ": proccess g + up -> dn + en + el^+"
      if (pid.eq. 8) print *,
     &  "pid = ", pid, ": proccess g + anti-dn -> anti-up + en + el^+"
      if (pid.eq. 9) print *,
     &  "pid = ", pid, ": proccess g + dn -> up + el^- + en"
      if (pid.eq.10) print *,
     &  "pid = ", pid, ": proccess g + anti-up -> anti-dn + el^- + en"
      if (pid.eq.11) print *,
     &  "pid = ", pid, ": proccess g + up -> dn + mn + mo^+"
      if (pid.eq.12) print *,
     &  "pid = ", pid, ": proccess g + anti-dn -> anti-up + mn + mo^+"
      if (pid.eq.13) print *,
     &  "pid = ", pid, ": proccess g + dn -> up + mo^- + mn"
      if (pid.eq.14) print *,
     &  "pid = ", pid, ": proccess g + anti-up -> anti-dn + mo^- + mn"
      if (pid.eq.15) print *,
     &  "pid = ", pid, ": proccess g + up -> dn + tn + ta^+"
      if (pid.eq.16) print *,
     &  "pid = ", pid, ": proccess g + anti-dn -> anti-up + tn + ta^+"
      if (pid.eq.17) print *,
     &  "pid = ", pid, ": proccess g + dn -> up + ta^- + tn"
      if (pid.eq.18) print *,
     &  "pid = ", pid, ": proccess g + anti-up -> anti-dn + ta^- + tn"

      return
      end
************************************************************************
